const babelPluginSyntaxFunctionBind = require('babel-plugin-syntax-function-bind')

const clean = (node) => {
  node.isClean = true
  return node
}

const toLogical = (t, path, callee) => {
  if (t.isMemberExpression(callee)) {
    path.replaceWith(
      t.logicalExpression(
        '&&',
        callee,
        clean(path.node)
      )
    )
  }
}

const isSafeUnsafeBind = (t, path) => [
  t.isIdentifier(path.node.object, { name: 'safe' }),
  t.isIdentifier(path.node.object, { name: 'unsafe' })
]

const ttt = t => ({
  BindExpression (path) {
    if (path.node.isClean) {
      return
    }
    const [isMayBeUndef, isWontBeUndef] = isSafeUnsafeBind(t, path)
    if (isMayBeUndef || isWontBeUndef) {
      path.skip()
    } else {
      toLogical(t, path, path.node.callee)
    }
  },
  MemberExpression (path) {
    if (this.shouldTranspile) {
      if (path.node.isClean) {
        return
      }
      const object = path.node.object
      path.replaceWith(
        t.logicalExpression(
          '&&',
          object,
          clean(path.node)
        )
      )
    }
  },
  AssignmentExpression (path) {
    if (this.shouldTranspile) {
      if (path.node.isClean) {
        return
      }
      const { operator, left, right } = path.node
      path.replaceWith(
        clean(t.assignmentExpression(
          operator,
          clean(left),
          right
        ))
      )
    }
  },
  CallExpression (path) {
    if (this.shouldTranspile) {
      if (path.node.isClean) {
        return
      }
      toLogical(t, path, path.node.callee)
    }
  }
})

module.exports = (babel) => {
  const t = babel.types
  return {
    inherits: babelPluginSyntaxFunctionBind,
    visitor: {
      BindExpression (path) {
        if (path.node.isClean) {
          return
        }
        const [isMayBeUndef, isWontBeUndef] = isSafeUnsafeBind(t, path)
        if (isMayBeUndef || isWontBeUndef) {
          if (isMayBeUndef) {
            path.traverse(ttt(t), { shouldTranspile: true })
          } else if (isWontBeUndef) {
            path.traverse(ttt(t), { shouldTranspile: false })
          }
          path.replaceWith(
            path.node.callee
          )
        } else {
          toLogical(t, path, path.node.callee)
        }
      }
    }
  }
}
