const babel = require('babel-core')
const accessUndefiend = require('../src/index')

const create = (func) => {
  const out = babel.transform(func.toString(), {
    plugins: [accessUndefiend]
  })
  const code = `return (${out.code})()`
  return new Function(code) // eslint-disable-line no-new-func
}

describe('access-undefined', () => {
  describe('objects', () => {
    it('returns existing value', () => {
      const result = create(function test () { // eslint-disable-line prefer-arrow-callback
        const obj = { a: 5, b: 7 }
        return safe::[obj.a, obj.b]
      })()
      expect(result).toEqual([5, 7])
    })

    it('returns nonexisting value as undefined', () => {
      const result = create(function test () { // eslint-disable-line prefer-arrow-callback
        const obj = { a: 5, b: 7 }
        return safe::[obj.c, obj.d]
      })()
      expect(result).toEqual([undefined, undefined])
    })

    it('prevents from an error returning undefined', () => {
      const result = create(function test () { // eslint-disable-line prefer-arrow-callback
        const obj = { a: 5, b: 7 }
        return safe::[obj.c.d, obj.e.r, obj.t.g.b.k, obj[5].ww[2]]
      })()
      expect(result).toEqual([undefined, undefined, undefined, undefined])
    })
  })

  describe('arrays', () => {
    it('returns existing value', () => {
      const result = create(function test () { // eslint-disable-line prefer-arrow-callback
        const arr = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        return safe::[arr[4], arr[8], arr[0]]
      })()
      expect(result).toEqual([4, 8, 0])
    })

    it('returns nonexisting value as undefined', () => {
      const result = create(function test () { // eslint-disable-line prefer-arrow-callback
        const arr = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        return safe::[arr[10], arr[13], arr[59]]
      })()
      expect(result).toEqual([undefined, undefined, undefined])
    })

    it('prevents from an error returning undefined', () => {
      const result = create(function test () { // eslint-disable-line prefer-arrow-callback
        const arr = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, [10, 11, 12]]
        return safe::[arr[5][6][1], arr[10][0][1][2], arr[22][22][22]]
      })()
      expect(result).toEqual([undefined, undefined, undefined])
    })
  })

  describe('general', () => {
    it('skips Assignment Expression', () => {
      const result = create(function test () { // eslint-disable-line prefer-arrow-callback
        const obj = {
          a: 5
        }
        obj.a = 6
        return obj.a
      })()
      expect(result).toEqual(6)
    })

    it('executes function only if callee is defined', () => {
      const result = create(function test () { // eslint-disable-line prefer-arrow-callback
        const obj = {
          a: () => 5
        }
        return safe::[obj.a(), obj.b()]
      })()
      expect(result).toEqual([5, undefined])
    })

    it('unsafe:: prevents from transpiling', () => {
      const result = () => create(function test () { // eslint-disable-line prefer-arrow-callback
        const arr = [1, 2, 3, 4, 5]
        const obj = safe::({
          a: arr[4][5][6],
          b: unsafe::arr[1][2][3]
        })
      })()
      expect(result).toThrow(new TypeError('Cannot read property \'3\' of undefined'))
    })
  })
})
